if not LibStub then return end

local icon = LibStub('LibDBIcon-1.0')

-- upvalues
local CalendarGetDate = CalendarGetDate
local CalendarGetEventInfo = CalendarGetEventInfo
local CalendarGetNumDayEvents = CalendarGetNumDayEvents
local CalendarSetMonth = CalendarSetMonth
local EasyMenu = EasyMenu
local OpenCalendar = OpenCalendar

local L = L

-- cache for events
local todaysEvents = {}
local upcomingEvents = {}

obj = LibStub:GetLibrary('LibDataBroker-1.1'):NewDataObject('Broker_Calendar', {
	type = 'data source',
	text = 'Calendar',
	icon = 'Interface\\Icons\\spell_holy_borrowedtime',
})
local obj = obj
local frame = CreateFrame('Frame')
frame:SetScript('OnEvent', function(self, event, ...) if self[event] then return self[event](self, event, ...) end end)
frame:RegisterEvent('PLAYER_ENTERING_WORLD')
frame:RegisterEvent('CALENDAR_UPDATE_EVENT_LIST')
frame:RegisterEvent('CALENDAR_UPDATE_INVITE_LIST')


local function ToggleMinimap()
	local hide = not CalendarDB.minimap.hide
	CalendarDB.minimap.hide = hide
	if hide then
		icon:Hide('Broker_Calendar')
	else
		icon:Show('Broker_Calendar')
	end
end

local function printTwoCharNumber(number)
  if number < 10 then
    return '0'..number
  else
    return number
  end
end

local function printDays(num)
  local returnString = '('..num..' '

  if num == 1 then
    returnString = ' '..returnString..L['DAY']..')'
  else
    returnString = returnString..L['DAYS']..')'
  end

  if num < 10 then
    returnString = '  '..returnString
  end
  return returnString
end

local function populateEvents(day, month)
  -- today
  local numEvents = CalendarGetNumDayEvents(0, day)
  if numEvents > 0 then
    local title, hour, minute
    for eventIndex = 1, numEvents do
      title, hour, minute, calendarType, _, _, _, _, _, invitedBy = CalendarGetDayEvent(0, day, eventIndex)
      if calendarType ~= 'RAID_LOCKOUT' and todaysEvents[title] == nil then
        todaysEvents[title] = {hour, minute}
      end
    end
  end

  -- upcoming
  local numEvents, title, days
  local monthOffset = 0
  local daysInMonth = select(3, CalendarGetMonth())
  for dayOffset = 1, 7 do
    if day + dayOffset > daysInMonth then
      monthOffset = monthOffset + 1
      day = 1 - dayOffset
      daysInMonth = select(3, CalendarGetMonth(monthOffset))
    end

    numEvents = CalendarGetNumDayEvents(monthOffset, day + dayOffset)
    if numEvents ~= 0 then
      for eventIndex = 1, numEvents do
        title, hour, minute, calendarType, _, _, _, _, _, invitedBy, _, _, _, days  = CalendarGetDayEvent(monthOffset, day + dayOffset, eventIndex)

        if upcomingEvents[title] == nil and todaysEvents[title] == nil and calendarType ~= 'RAID_LOCKOUT' then
          upcomingEvents[title] = {day + dayOffset, month + monthOffset, days}
        end
      end
    end
  end

  local sortedEvents = {}
  for title, time in pairs(todaysEvents) do
    table.insert(sortedEvents, {title = title, hour = time[1], minute = time[2]})
  end

  if #sortedEvents > 1 then
    -- sort by start time
    table.sort(sortedEvents, function(a, b)
      if a.hour == nil or b.hour == nil then
        return
      elseif a.hour == b.hour then
        if a.minute == b.minute then
          return a.title < b.title
        else
          return a.minute < b.minute
        end
      else
        return a.hour < b.hour
      end
    end)
  end

  todaysEvents = sortedEvents

  sortedEvents = {}
  for title, start in pairs(upcomingEvents) do
    table.insert(sortedEvents, {title = title, day = start[1], month = start[2], duration = start[3]})
  end

  if #sortedEvents > 1 then
    -- sort by start time
    table.sort(sortedEvents, function(a, b)
      if a.month == nil or b.month == nil then
        return
      elseif a.month == b.month then
        if a.day == b.day then
          return a.title < b.title
        else
          return a.day < b.day
        end
      else
        return a.month < b.month
      end
    end)
  end

  upcomingEvents = sortedEvents
  sortedEvents = nil
end

local function reloadEvents()
  --ChatFrame1:AddMessage(#todaysEvents..' - '..#upcomingEvents)
  todaysEvents = nil
  upcomingEvents = nil
  todaysEvents = {}
  upcomingEvents = {}

  local _, month, day = CalendarGetDate()
  populateEvents(day, month)
  --ChatFrame1:AddMessage(day..'.'..month..' '..#todaysEvents..' - '..#upcomingEvents)
end

local function showTodaysEvents()
  if #todaysEvents == 0 then
    GameTooltip:AddLine(L['NO_EVENTS_TODAY'])
  else
    GameTooltip:AddDoubleLine(L['TODAY']..':', L['STARTS'], 0.9, 0.6, 0.2, 0.9, 0.6, 0.2)

    for i = 1, #todaysEvents do
      if todaysEvents[i].hour ~= nil then
        GameTooltip:AddDoubleLine(todaysEvents[i].title, printTwoCharNumber(todaysEvents[i].hour)..':'..printTwoCharNumber(todaysEvents[i].minute), 1, 1, 0, 0.2, 1, 0.2)
      end
    end
  end
end

local function showUpcomingEvents()
  if #upcomingEvents > 0 then
    GameTooltip:AddLine(' ')
    GameTooltip:AddDoubleLine(L['COMMING_SOON']..':', L['STARTS']..' ('..L['LASTS']..')', 0.9, 0.6, 0.2, 0.9, 0.6, 0.2)

    for i = 1, #upcomingEvents do
      if upcomingEvents[i].day ~= nil then
        GameTooltip:AddDoubleLine(upcomingEvents[i].title, printTwoCharNumber(upcomingEvents[i].day)..'.'..printTwoCharNumber(upcomingEvents[i].month)..'. '..printDays(upcomingEvents[i].duration), 1, 1, 0, 0.2, 1, 0.2)
      end
    end
  end
end

-- All credit for this func goes to Tekkub and his picoGuild!
local function GetTipAnchor(frame)
  local x, y = frame:GetCenter()
  if not x or not y then return 'TOPLEFT', 'BOTTOMLEFT' end
  local hhalf = (x > UIParent:GetWidth() * 2 / 3) and 'RIGHT' or (x < UIParent:GetWidth() / 3) and 'LEFT' or ''
  local vhalf = (y > UIParent:GetHeight() / 2) and 'TOP' or 'BOTTOM'
  return vhalf..hhalf, frame, (vhalf == 'TOP' and 'BOTTOM' or 'TOP')..hhalf
end

local function toggleMinimap()
  local hide = not CalendarDB.minimap.hide
	CalendarDB.minimap.hide = hide
	if hide then
		icon:Hide('Broker_Calendar')
	else
		icon:Show('Broker_Calendar')
	end
end

local function getMenu()
  local menu = {}
  local entry = {}
  -- title
  entry.text = 'Broker Calendar - '..L['OPTIONS']
  entry.isTitle = true
  menu[1] = entry
  
  -- separator
  entry = {}
  entry.text = ''
  entry.disabled = true
  entry.separator = true
  menu[2] = entry
  
  -- show on minimap
  entry = {}
  entry.text = L['ATT_MINIMAP']
  entry.checked = not CalendarDB.minimap.hide
  entry.func = toggleMinimap
  menu[3] = entry
  
  return menu
end

local function getMenuFrame(self)
  frame:SetPoint(GetTipAnchor(self))
  frame.displayMode = 'MENU'
  frame.relativeTo = self
  return frame
end

function frame:PLAYER_ENTERING_WORLD()
	-- PortalsDB.minimap is there for smooth upgrade of SVs from old version
	if not CalendarDB then
		CalendarDB = {}
		CalendarDB.minimap = {}
		CalendarDB.minimap.hide = false
		CalendarDB.version = 1
	end

	if icon then
		icon:Register('Broker_Calendar', obj, CalendarDB.minimap)
	end

	self:UnregisterEvent('PLAYER_ENTERING_WORLD')

  self.counter, self.delay = 0, 60
  local _, month, day = CalendarGetDate()
  frame:SetScript('OnUpdate', function(self, elapsed)
    self.counter = self.counter + elapsed
    if self.counter < self.delay then
      OpenCalendar()
      CalendarSetMonth(0)
      populateEvents(day, month)
      if #todaysEvents > 0 or #upcomingEvents > 0 then
        frame:SetScript('OnUpdate', nil)
      end
      self.counter = 0
    end
  end)
end

function frame:CALENDAR_UPDATE_EVENT_LIST()
  reloadEvents()
end
function frame:CALENDAR_UPDATE_INVITE_LIST()
  reloadEvents()
end

function obj.OnClick(self, button)
  GameTooltip:Hide()
  if button == 'RightButton' then
    EasyMenu(getMenu(), getMenuFrame(self))
  else
    GameTimeFrame_OnClick(GameTimeFrame)
  end
end

function obj.OnLeave()
	GameTooltip:Hide() 
end

function obj.OnEnter(self)
 	GameTooltip:SetOwner(self, 'ANCHOR_NONE')
	GameTooltip:SetPoint(GetTipAnchor(self))
	GameTooltip:ClearLines()

  GameTooltip:AddLine('Broker Calendar', 1, 1, 0)
  local weekday, month, day, year = CalendarGetDate()
  GameTooltip:AddLine(printTwoCharNumber(day)..'. '..printTwoCharNumber(month)..'. '..printTwoCharNumber(year), 1, 1, 0)
  GameTooltip:AddLine(' ')

  showTodaysEvents()
  showUpcomingEvents()

	GameTooltip:Show()
end

-- slashcommand definition
SlashCmdList['BROKER_CALENDAR'] = function() ToggleMinimap() end
SLASH_BROKER_CALENDAR1 =  '/b_calendar'